// ignore_for_file: public_member_api_docs

import 'dart:math';

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';

part 'channel_with_pivot.g.dart';

/// [ChannelWithPivot] is a widget drawing an ecg sequence per channel.
@CopyWith()
class ChannelWithPivot extends StatelessWidget {
  /// [ChannelWithPivot] is a widget that draw a graph of
  /// an ecg sequence by a given [ecgSequence].
  const ChannelWithPivot({
    super.key,
    required this.ecgSequence,
    this.samplingRate = 250,
    this.strokeWidth = 1.5,
    this.color = Colors.black,
    this.measureTime = 60,
    this.maxY = 2,
    this.minY = -2,
    this.pivot = 0,
    this.pivotColor = Colors.transparent,
    this.pivotStrokeWidth = 0,
    this.backgroundColor = Colors.transparent,
    this.isTappable = false,
    this.countRemoves = 125,
    this.countUpdates = 9,
  });

  final double measureTime;
  final double strokeWidth;
  final int samplingRate;
  final List<FlSpot> ecgSequence;

  final int pivot;
  final Color pivotColor;
  final double pivotStrokeWidth;

  final int countRemoves;
  final int countUpdates;

  final Color backgroundColor;

  final double maxY;
  final double minY;

  final Color color;
  final bool isTappable;

  @override
  Widget build(BuildContext context) {
    if (ecgSequence.isEmpty) return const SizedBox();
    final tic = samplingRate * measureTime;

    const minX = 0.0;
    final maxX = tic;

    return LineChart(
      LineChartData(
        minX: minX,
        maxX: maxX,
        minY: minY,
        maxY: maxY,
        backgroundColor: backgroundColor,
        lineTouchData: LineTouchData(enabled: false),
        gridData: FlGridData(show: false),
        titlesData: FlTitlesData(show: false),
        borderData: FlBorderData(show: false),
        lineBarsData: [
          LineChartBarData(
            // spots: ecgSequence,
            spots: ecgSequence
                .getRange(0, pivot)
                // .where((e) => e.x.round() % 10 == 0)
                .toList(growable: false),
            color: color,
            isCurved: false,
            barWidth: strokeWidth,
            dotData: const FlDotData(show: false),
          ),
          if (pivot + countRemoves + 1 < ecgSequence.length + countRemoves / 2)
            LineChartBarData(
              spots: ecgSequence
                  .getRange(
                    pivot,
                    min(pivot + countRemoves + 1, ecgSequence.length),
                  )
                  // .where((e) => e.x.round() % 10 == 0)
                  .toList(growable: false),
              isCurved: false,
              barWidth: strokeWidth,
              dotData: const FlDotData(show: false),
              gradient: LinearGradient(
                colors: [color.withOpacity(0), color],
                stops: const [0.4, 1],
              ),
            ),
          if (pivot + countRemoves < ecgSequence.length)
            LineChartBarData(
              spots: ecgSequence
                  .getRange(
                    pivot + countRemoves,
                    ecgSequence.length,
                  )
                  // .where((e) => e.x.round() % 10 == 0)
                  .toList(growable: false),
              color: color,
              isCurved: false,
              barWidth: strokeWidth,
              dotData: const FlDotData(show: false),
            ),
        ],
      ),
      duration: Duration(milliseconds: 16),
      curve: accelerateEasing,
      // key: ValueKey(pivot),
      // chartRendererKey: ValueKey(pivot),
    );
  }
}
